--
-- ͻ
--        D E S I G N   E N G I N E E R I N G            DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         QtOpenCVAda - Ada to Qt6 and OpenCV interface              
--                                                                    
--         Author :  Leonid Dulman     2021,2022                      
--  Copyright (C) 2012,2022                                           
-- ͼ
--
--   This library is free software; you can redistribute it and/or modify  *
--   it under the terms of the GNU Library General Public License as       *
--   published by the Free Software Foundation; either version 3 of the    *
--   License, or (at your option) any later version.                       *

with Qt.QIntList; use Qt.QIntList;
with Qt.QString;  use Qt.QString; 
with Interfaces.C;use Interfaces.C;
package Qt.OpenCV is

 type QCVStringH            is new QObjectH;
 type QCVSizeH              is new QObjectH;
 type QCVScalarH            is new QObjectH;
 type QCVRectH              is new QObjectH;
 type QCVPointH             is new QObjectH;
 type QCVPointsH            is new QObjectH;
 type QCVPoint2fH           is new QObjectH;
 type QCVOutputArrayH       is new QObjectH;
 type QCVInputArrayOfArraysH is new QObjectH;
 type QCVVideoWriterH       is new QObjectH;
 type QCVNetH               is new QObjectH;
 type QCVMatH               is new QObjectH;
 type QCVUMatH              is new QObjectH;
 type QCVInputArrayH        is new QObjectH;
 type QCVVideoCaptureH      is new QObjectH;
 type QCVCaptureH           is new QObjectH;
 type QCVVideoRecieverH     is new QThreadH;
 type QCVVideoTransmitterH  is new QObjectH;
 type QtOpenCVViewerGlH     is new QOpenGLWidgetH;
 type QCVFpsCheckerH        is new QObjectH;
 type QCVCascadeClassifierH is new QObjectH;
 type QCVFacesH             is new QObjectH;
 type QCVImagesH            is new QObjectH;
 type QCVLabelsH            is new QObjectH;
 type QCVLandmarksH         is new QObjectH;
 type QCVWeightsH           is new QObjectH;
 type QCVFileNodeH          is new QObjectH;
 type QCVFaceMarkH          is new QObjectH;
 type QCVPoints2fH          is new QObjectH;
 
 type QCVTessBaseAPIH          is new QObjectH;
 type QCVDictFuncH             is new QObjectH;
 type QCVProbabilityInContextFuncH is new QObjectH;
 type QCVDawgH                 is new QObjectH;
 type QCVOSResultsH            is new QObjectH;
 type QCVTesseractH            is new QObjectH;
 type QCVETEXT_DESCH           is new QObjectH;
 type QCVPixH                  is new QObjectH; 
 type QCVBoxaH                 is new QObjectH;
 type QCVPageIteratorH         is new QObjectH;
 type QCVResultIteratorH       is new QObjectH;
 type QCVTessResultRendererH   is new QObjectH;
 type QCVMutableIteratorH      is new QObjectH;
 type QCVFILEH                 is new QObjectH;
 type QCVFileReaderH           is new QObjectH;
 type QCVExceptionH            is new QObjectH;
 type SparseMatH               is new QObjectH;
 type QCVRNGH                  is new QObjectH;
 type QCVTermCriteriaH         is new QObjectH;
 type QCVMatListH              is new QObjectListH; -- QObjectH <-> QCVMatH;
 type QCVRectListH             is new QObjectListH;
--------------------------------------------------------------------
 type conversion_type is new integer;

 CV_8UC1   : constant conversion_type :=   0;
 CV_8UC2   : constant conversion_type :=   8;
 CV_8UC3   : constant conversion_type :=  16;
 CV_8UC4   : constant conversion_type :=  24;

 CV_8SC1   : constant conversion_type :=   1;
 CV_8SC2   : constant conversion_type :=   9;
 CV_8SC3   : constant conversion_type :=  17;
 CV_8SC4   : constant conversion_type :=  25;

 CV_16UC1  : constant conversion_type :=   2;
 CV_16UC2  : constant conversion_type :=  10;
 CV_16UC3  : constant conversion_type :=  18;
 CV_16UC4  : constant conversion_type :=  26;

 CV_16SC1  : constant conversion_type :=   3;
 CV_16SC2  : constant conversion_type :=  11;
 CV_16SC3  : constant conversion_type :=  19;
 CV_16SC4  : constant conversion_type :=  27;

 CV_32SC1  : constant conversion_type :=   4;
 CV_32SC2  : constant conversion_type :=  12;
 CV_32SC3  : constant conversion_type :=  20;
 CV_32SC4  : constant conversion_type :=  28;

 CV_32FC1  : constant conversion_type :=   5;
 CV_32FC2  : constant conversion_type :=  13;
 CV_32FC3  : constant conversion_type :=  21;
 CV_32FC4  : constant conversion_type :=  29;

 CV_64FC1  : constant conversion_type :=   6;
 CV_64FC2  : constant conversion_type :=  14;
 CV_64FC3  : constant conversion_type :=  22;
 CV_64FC4  : constant conversion_type :=  30;

 CV_16FC1  : constant conversion_type :=   7;
 CV_16FC2  : constant conversion_type :=  15;
 CV_16FC3  : constant conversion_type :=  23;
 CV_16FC4  : constant conversion_type :=  31;
 
 CV_8U     : constant := 0;
 CV_8S     : constant := 1;
 CV_16U    : constant := 2;
 CV_16S    : constant := 3;
 CV_32S    : constant := 4;
 CV_32F    : constant := 5;
 CV_64F    : constant := 6;
 CV_16F    : constant := 7;
--------------------------------------------------------------------------------------------------------------------------
 DBL_MAX   : constant double := 1.79769e+308;
--------------------------------------------------------------------------------------------------------------------------
 
 type ImreadModes is new integer;
 IMREAD_UNCHANGED           :constant ImreadModes := -1; -- If set, return the loaded image as is (with alpha channel, otherwise it gets cropped).
 IMREAD_GRAYSCALE           :constant ImreadModes :=  0; -- If set, always convert image to the single channel grayscale image (codec internal conversion).
 IMREAD_COLOR               :constant ImreadModes :=  1; -- If set, always convert image to the 3 channel BGR color image.
 IMREAD_ANYDEPTH            :constant ImreadModes :=  2; -- If set, return 16-bit/32-bit image when the input has the corresponding depth, otherwise convert it to 8-bit.
 IMREAD_ANYCOLOR            :constant ImreadModes :=  4; -- If set, the image is read in any possible color format.
 IMREAD_LOAD_GDAL           :constant ImreadModes :=  8; -- If set, use the gdal driver for loading the image.
 IMREAD_REDUCED_GRAYSCALE_2 :constant ImreadModes := 16; -- If set, always convert image to the single channel grayscale image and the image size reduced 1/2.
 IMREAD_REDUCED_COLOR_2     :constant ImreadModes := 17; -- If set, always convert image to the 3 channel BGR color image and the image size reduced 1/2.
 IMREAD_REDUCED_GRAYSCALE_4 :constant ImreadModes := 32; -- If set, always convert image to the single channel grayscale image and the image size reduced 1/4.
 IMREAD_REDUCED_COLOR_4     :constant ImreadModes := 33; -- If set, always convert image to the 3 channel BGR color image and the image size reduced 1/4.
 IMREAD_REDUCED_GRAYSCALE_8 :constant ImreadModes := 64; -- If set, always convert image to the single channel grayscale image and the image size reduced 1/8.
 IMREAD_REDUCED_COLOR_8     :constant ImreadModes := 65; -- If set, always convert image to the 3 channel BGR color image and the image size reduced 1/8.
 IMREAD_IGNORE_ORIENTATION  :constant ImreadModes := 128;-- If set, do not rotate the image according to EXIF's orientation flag.
 
 type ImwriteFlags is new integer;
 IMWRITE_JPEG_QUALITY               : constant ImwriteFlags :=   1; -- For JPEG, it can be a quality from 0 to 100 (the higher is the better). Default value is 95.
 IMWRITE_JPEG_PROGRESSIVE           : constant ImwriteFlags :=   2; -- Enable JPEG features, 0 or 1, default is False.
 IMWRITE_JPEG_OPTIMIZE              : constant ImwriteFlags :=   3; -- Enable JPEG features, 0 or 1, default is False.
 IMWRITE_JPEG_RST_INTERVAL          : constant ImwriteFlags :=   4; -- JPEG restart interval, 0 - 65535, default is 0 - no restart.
 IMWRITE_JPEG_LUMA_QUALITY          : constant ImwriteFlags :=   5; -- Separate luma quality level, 0 - 100, default is 0 - don't use.
 IMWRITE_JPEG_CHROMA_QUALITY        : constant ImwriteFlags :=   6; -- Separate chroma quality level, 0 - 100, default is 0 - don't use.
 IMWRITE_PNG_COMPRESSION            : constant ImwriteFlags :=  16; -- For PNG, it can be the compression level from 0 to 9. A higher value means a smaller size and longer compression time. If specified, strategy is changed to IMWRITE_PNG_STRATEGY_DEFAULT (Z_DEFAULT_STRATEGY). Default value is 1 (best speed setting).
 IMWRITE_PNG_STRATEGY               : constant ImwriteFlags :=  17; -- One of cv::ImwritePNGFlags, default is IMWRITE_PNG_STRATEGY_RLE.
 IMWRITE_PNG_BILEVEL                : constant ImwriteFlags :=  18; -- Binary level PNG, 0 or 1, default is 0.
 IMWRITE_PXM_BINARY                 : constant ImwriteFlags :=  32; -- For PPM, PGM, or PBM, it can be a binary format flag, 0 or 1. Default value is 1.
 IMWRITE_EXR_TYPE                   : constant ImwriteFlags :=  48; -- override EXR storage type (FLOAT (FP32) is default)
 IMWRITE_EXR_COMPRESSION            : constant ImwriteFlags :=  49; -- override EXR compression type (ZIP_COMPRESSION = 3 is default)
 IMWRITE_WEBP_QUALITY               : constant ImwriteFlags :=  64; -- For WEBP, it can be a quality from 1 to 100 (the higher is the better). By default (without any parameter) and for quality above 100 the lossless compression is used.
 IMWRITE_PAM_TUPLETYPE              : constant ImwriteFlags := 128; -- For PAM, sets the TUPLETYPE field to the corresponding string value that is defined for the format
 IMWRITE_TIFF_RESUNIT               : constant ImwriteFlags := 256; -- For TIFF, use to specify which DPI resolution unit to set; see libtiff documentation for valid values
 IMWRITE_TIFF_XDPI                  : constant ImwriteFlags := 257; -- For TIFF, use to specify the X direction DPI
 IMWRITE_TIFF_YDPI                  : constant ImwriteFlags := 258; -- For TIFF, use to specify the Y direction DPI
 IMWRITE_TIFF_COMPRESSION           : constant ImwriteFlags := 259; -- For TIFF, use to specify the image compression scheme. See libtiff for integer constants corresponding to compression formats. Note, for images whose depth is CV_32F, only libtiff's SGILOG compression scheme is used. For other supported depths, the compression scheme can be specified by this flag; LZW compression is the default.
 IMWRITE_JPEG2000_COMPRESSION_X1000 : constant ImwriteFlags := 272; -- For JPEG2000, use to specify the target compression rate (multiplied by 1000). The value can be from 0 to 1000. Default is 1000.

 subtype ImwriteEXRTypeFlags is ImwriteFlags;
 IMWRITE_EXR_TYPE_HALF : constant ImwriteEXRTypeFlags := 1; -- store as HALF (FP16)
 IMWRITE_EXR_TYPE_FLOAT: constant ImwriteEXRTypeFlags := 2; -- store as FP32 (default)

 subtype ImwriteEXRCompressionFlags is ImwriteFlags;
 IMWRITE_EXR_COMPRESSION_NO    : constant ImwriteEXRCompressionFlags := 0; -- no compression
 IMWRITE_EXR_COMPRESSION_RLE   : constant ImwriteEXRCompressionFlags := 1; -- run length encoding
 IMWRITE_EXR_COMPRESSION_ZIPS  : constant ImwriteEXRCompressionFlags := 2; -- zlib compression, one scan line at a time
 IMWRITE_EXR_COMPRESSION_ZIP   : constant ImwriteEXRCompressionFlags := 3; -- zlib compression, in blocks of 16 scan lines
 IMWRITE_EXR_COMPRESSION_PIZ   : constant ImwriteEXRCompressionFlags := 4; -- piz-based wavelet compression
 IMWRITE_EXR_COMPRESSION_PXR24 : constant ImwriteEXRCompressionFlags := 5; -- lossy 24-bit float compression
 IMWRITE_EXR_COMPRESSION_B44   : constant ImwriteEXRCompressionFlags := 6; -- lossy 4-by-4 pixel block compression, fixed compression rate
 IMWRITE_EXR_COMPRESSION_B44A  : constant ImwriteEXRCompressionFlags := 7; -- lossy 4-by-4 pixel block compression, flat fields are compressed more
 IMWRITE_EXR_COMPRESSION_DWAA  : constant ImwriteEXRCompressionFlags := 8; -- lossy DCT based compression, in blocks of 32 scanlines. More efficient for partial buffer access.
 IMWRITE_EXR_COMPRESSION_DWAB  : constant ImwriteEXRCompressionFlags := 9; -- lossy DCT based compression, in blocks of 256 scanlines. More efficient space wise and faster to decode full frames than DWAA_COMPRESSION.

 subtype ImwritePNGFlags is ImwriteFlags;
 IMWRITE_PNG_STRATEGY_DEFAULT      : constant ImwritePNGFlags := 0; -- Use this value for normal data.
 IMWRITE_PNG_STRATEGY_FILTERED     : constant ImwritePNGFlags := 1; -- Use this value for data produced by a filter (or predictor).Filtered data consists mostly of small values with a somewhat random distribution. In this case, the compression algorithm is tuned to compress them better.
 IMWRITE_PNG_STRATEGY_HUFFMAN_ONLY : constant ImwritePNGFlags := 2; -- Use this value to force Huffman encoding only (no string match).
 IMWRITE_PNG_STRATEGY_RLE          : constant ImwritePNGFlags := 3; -- Use this value to limit match distances to one (run-length encoding).
 IMWRITE_PNG_STRATEGY_FIXED        : constant ImwritePNGFlags := 4; -- Using this value prevents the use of dynamic Huffman codes, allowing for a simpler decoder for special applications.

 -- Imwrite PAM specific tupletype flags used to define the 'TUPETYPE' field of a PAM file.
 subtype ImwritePAMFlags is ImwriteFlags;
 IMWRITE_PAM_FORMAT_NULL            : constant ImwritePAMFlags := 0;
 IMWRITE_PAM_FORMAT_BLACKANDWHITE   : constant ImwritePAMFlags := 1;
 IMWRITE_PAM_FORMAT_GRAYSCALE       : constant ImwritePAMFlags := 2;
 IMWRITE_PAM_FORMAT_GRAYSCALE_ALPHA : constant ImwritePAMFlags := 3;
 IMWRITE_PAM_FORMAT_RGB             : constant ImwritePAMFlags := 4;
 IMWRITE_PAM_FORMAT_RGB_ALPHA       : constant ImwritePAMFlags := 5;
 ---------------------------------------------------------------------------------------------------------------------------
 function  QOpenCV_imread(fileName: not null QStringH;flags : ImreadModes :=  IMREAD_COLOR ) return QCVMatH;
 pragma Import(C,QOpenCV_imread, "QOpenCV_imread");
 
 function  QOpenCV_imreadmulti(fileName: not null QStringH;mats:QCVMatH;flags : ImreadModes :=  IMREAD_ANYCOLOR) return  boolean;
 pragma Import(C,QOpenCV_imreadmulti, "QOpenCV_imreadmulti");
 
 function  QOpenCV_imdecode(buf: QCVMatH;flags : ImreadModes) return QCVMatH;
 pragma Import(C,QOpenCV_imdecode, "QOpenCV_imdecode");
 
 function  QOpenCV_imdecode2(buf:QCVMatH;flags: ImreadModes;dst: QCVMatH) return QCVMatH;
 pragma Import(C,QOpenCV_imdecode2, "QOpenCV_imdecode2");
 
 function  QOpenCV_imencode(ext:QStringH;img: QCVMatH;buf:QCVMatListH;params :QIntListH := QIntList_create ) return  boolean;
 pragma Import(C,QOpenCV_imencode, "QOpenCV_imencode");
 
 function  QOpenCV_haveImageReader(fileName: not null QStringH) return  boolean;
 pragma Import(C,QOpenCV_haveImageReader, "QOpenCV_haveImageReader");
 
 function  QOpenCV_haveImageWriter(fileName: not null QStringH) return  boolean;
 pragma Import(C,QOpenCV_haveImageWriter, "QOpenCV_haveImageWriter");
 
 procedure QOpenCV_imshow(handle:not null QCVMatH;imShow:not null QLabelH);
 pragma Import(C,QOpenCV_imshow, "QOpenCV_imshow");
 
 ----------------------------------------------------------------------------------------
 
 function  QCVFaces_create return QCVFacesH;
 pragma Import(C,QCVFaces_create, "QCVFaces_create");
 
 function  QCVFaces_size(handle:not null QCVFacesH) return integer;
 pragma Import(C,QCVFaces_size, "QCVFaces_size");
 
 procedure QCVFaces_append(handle:not null QCVFacesH;frame:QCVRectH);
 pragma Import(C,QCVFaces_append, "QCVFaces_append");
  
 procedure QCVFaces_copyTo(handle:not null QCVFacesH;frame: not null QCVMatH);
 pragma Import(C,QCVFaces_copyTo, "QCVFaces_copyTo");
 
 procedure QCVFaces_at(handle: not null QCVFacesH;rect: not null QCVRectH;index:integer);
 pragma Import(C,QCVFaces_at, "QCVFaces_at");
 
 function  QCVLabels_create return QCVLabelsH;
 pragma Import(C,QCVLabels_create, "QCVLabels_create");
 
 function  QCVLabels_size(handle: not null QCVLabelsH) return integer;
 pragma Import(C,QCVLabels_size, "QCVLabels_size");
 
 function  QCVLabels_at(handle: not null QCVLabelsH;index:integer) return integer;
 pragma Import(C,QCVLabels_at, "QCVLabels_at");
  
 procedure QCVLabels_append(handle:not null QCVLabelsH;num:integer);
 pragma Import(C,QCVLabels_append, "QCVLabels_append");
 
 -----------------------------------------------------------------------
 function  QCVPoints_create return QCVPointsH;
 pragma Import(C,QCVPoints_create, "QCVPoints_create");
 
 function  QCVPoints_size(handle:not null QCVPointsH) return integer;
 pragma Import(C,QCVPoints_size, "QCVPoints_size");
 
 procedure QCVPoints_at(handle:not null QCVPointsH;point: not null QCVPoint2fH;index:integer);
 pragma Import(C,QCVPoints_at, "QCVPoints_at");
 
 procedure QCVPoints_append(handle:not null QCVPointsH;point: not null QCVPoint2fH);
 pragma Import(C,QCVPoints_append, "QCVPoints_append");
 

 -----------------------------------------------------------------------

 function  QCVLandmarks_create return QCVLandmarksH;
 pragma Import(C,QCVLandmarks_create, "QCVLandmarks_create");
 
 function  QCVLandmarks_size(handle: not null QCVLandmarksH) return integer;
 pragma Import(C,QCVLandmarks_size, "QCVLandmarks_size");
 
 procedure QCVLandmarks_at(handle: not null QCVLandmarksH;points:QCVPointsH;index:integer);
 pragma Import(C,QCVLandmarks_at, "QCVLandmarks_at");
 
 procedure QCVLandmarks_append(handle:not null QCVLandmarksH;points:QCVPointsH);
 pragma Import(C,QCVLandmarks_append, "QCVLandmarks_append");
 
 ------------------------------------------------------------------------------------
 function  QCVImages_create return QCVImagesH;
 pragma Import(C,QCVImages_create, "QCVImages_create");
 
 function  QCVImages_size(handle:not null QCVImagesH) return integer;
 pragma Import(C,QCVImages_size, "QCVImages_size");
 
 procedure QCVImages_at(handle: not null QCVImagesH;image: not null QCVMatH;index:integer);
 pragma Import(C,QCVImages_at, "QCVImages_at");
 
 procedure QCVImages_append(handle:not null QCVImagesH;image: not null QCVMatH);
 pragma Import(C,QCVImages_append, "QCVImages_append");
 
 ---------------------------------------------------------------------------------------
 function  QOpenCV_imdecode(buf:QCVMatH;flags: ImreadModes;dst: QCVMatH) return QCVMatH;
 procedure QOpenCV_imshow(frame: not null QCVMatH;graphicsView : not null QGraphicsViewH);

 function  QOpenCV_ConvertMatToQImage(frame: not null QCVMatH;enableDeepCopy : boolean :=  false) return QImageH;
 pragma Import(C,QOpenCV_ConvertMatToQImage, "QOpenCV_ConvertMatToQImage");
 
 function  QOpenCV_ConvertQImageToMat(image: not null QImageH;enableDeepCopy : boolean :=  false) return QCVMatH;
 pragma Import(C,QOpenCV_ConvertQImageToMat, "QOpenCV_ConvertQImageToMat");

 ----------------------------------------------------------------------------------------- 
 function  QCVCapture_RTSP(cvwin:QLabelH;rtsp_url :QStringH := s2qs("http://localhost:56000/mjpeg")) return integer;
 pragma Import(C,QCVCapture_RTSP, "QCVCapture_RTSP");
 
end;
