--
-- ͻ
--        D E S I G N   E N G I N E E R I N G            DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         QtOpenCVAda - Ada to Qt6 and OpenCV interface              
--                                                                    
--         Author :  Leonid Dulman     2022                           
--  Copyright (C) 2012,2022                                           
-- ͼ
--
--   This library is free software; you can redistribute it and/or modify  *
--   it under the terms of the GNU Library General Public License as       *
--   published by the Free Software Foundation; either version 3 of the    *
--   License, or (at your option) any later version.                       *

with Qt.OpenCV; use Qt.OpenCV;
with Qt.QCVCore; use Qt.QCVCore;
with Qt; use Qt;
with Qt.QString   ; use Qt.QString;
package Qt.QCVException  is

 type OpenCVErrorCode is new integer range -223 .. 0;
 StsOk                    : constant OpenCVErrorCode :=  0;  -- everything is ok
 StsBackTrace             : constant OpenCVErrorCode := -1;  -- pseudo error for back trace
 StsError                 : constant OpenCVErrorCode := -2;  -- unknown /unspecified error
 StsInternal              : constant OpenCVErrorCode := -3;  -- internal error (bad state)
 StsNoMem                 : constant OpenCVErrorCode := -4;  -- insufficient memory
 StsBadArg                : constant OpenCVErrorCode := -5;  -- function arg/param is bad
 StsBadFunc               : constant OpenCVErrorCode := -6;  -- unsupported function
 StsNoConv                : constant OpenCVErrorCode := -7;  -- iteration didn't converge
 StsAutoTrace             : constant OpenCVErrorCode := -8;  -- tracing
 HeaderIsNull             : constant OpenCVErrorCode := -9;  -- image header is NULL
 BadImageSize             : constant OpenCVErrorCode := -10;  -- image size is invalid
 BadOffset                : constant OpenCVErrorCode := -11;  -- offset is invalid
 BadDataPtr               : constant OpenCVErrorCode := -12;  --
 BadStep                  : constant OpenCVErrorCode := -13;  -- image step is wrong; this may happen for a non-continuous matrix.
 BadModelOrChSeq          : constant OpenCVErrorCode := -14;  --
 BadNumChannels           : constant OpenCVErrorCode := -15;  -- bad number of channels; for example; some functions accept only single channel matrices.
 BadNumChannel1U          : constant OpenCVErrorCode := -16;  --
 BadDepth                 : constant OpenCVErrorCode := -17;  -- input image depth is not supported by the function
 BadAlphaChannel          : constant OpenCVErrorCode := -18;  --
 BadOrder                 : constant OpenCVErrorCode := -19;  -- number of dimensions is out of range
 BadOrigin                : constant OpenCVErrorCode := -20;  -- incorrect input origin
 BadAlign                 : constant OpenCVErrorCode := -21;  -- incorrect input align
 BadCallBack              : constant OpenCVErrorCode := -22;  --
 BadTileSize              : constant OpenCVErrorCode := -23;  --
 BadCOI                   : constant OpenCVErrorCode := -24;  -- input COI is not supported
 BadROISize               : constant OpenCVErrorCode := -25;  -- incorrect input roi
 MaskIsTiled              : constant OpenCVErrorCode := -26;  --
 StsNullPtr               : constant OpenCVErrorCode := -27;  -- null pointer
 StsVecLengthErr          : constant OpenCVErrorCode := -28;  -- incorrect vector length
 StsFilterStructContentErr: constant OpenCVErrorCode := -29;  -- incorrect filter structure content
 StsKernelStructContentErr: constant OpenCVErrorCode := -30;  -- incorrect transform kernel content
 StsFilterOffsetErr       : constant OpenCVErrorCode := -31;  -- incorrect filter offset value
 StsBadSize               : constant OpenCVErrorCode := -201; -- the input/output structure size is incorrect
 StsDivByZero             : constant OpenCVErrorCode := -202; -- division by zero
 StsInplaceNotSupported   : constant OpenCVErrorCode := -203; -- in-place operation is not supported
 StsObjectNotFound        : constant OpenCVErrorCode := -204; -- request can't be completed
 StsUnmatchedFormats      : constant OpenCVErrorCode := -205; -- formats of input/output arrays differ
 StsBadFlag               : constant OpenCVErrorCode := -206; -- flag is wrong or not supported
 StsBadPoint              : constant OpenCVErrorCode := -207; -- bad CvPoint
 StsBadMask               : constant OpenCVErrorCode := -208; -- bad format of mask (neither 8uC1 nor 8sC1)
 StsUnmatchedSizes        : constant OpenCVErrorCode := -209; -- sizes of input/output structures do not match
 StsUnsupportedFormat     : constant OpenCVErrorCode := -210; -- the data format/type is not supported by the function
 StsOutOfRange            : constant OpenCVErrorCode := -211; -- some of parameters are out of range
 StsParseError            : constant OpenCVErrorCode := -212; -- invalid syntax/structure of the parsed file
 StsNotImplemented        : constant OpenCVErrorCode := -213; -- the requested function/feature is not implemented
 StsBadMemBlock           : constant OpenCVErrorCode := -214; -- an allocated block has been corrupted
 StsAssert                : constant OpenCVErrorCode := -215; -- assertion failed
 GpuNotSupported          : constant OpenCVErrorCode := -216; -- no CUDA support
 GpuApiCallError          : constant OpenCVErrorCode := -217; -- GPU API call error
 OpenGlNotSupported       : constant OpenCVErrorCode := -218; -- no OpenGL support
 OpenGlApiCallError       : constant OpenCVErrorCode := -219; -- OpenGL API call error
 OpenCLApiCallError       : constant OpenCVErrorCode := -220; -- OpenCL API call error
 OpenCLDoubleNotSupported : constant OpenCVErrorCode := -221;
 OpenCLInitError          : constant OpenCVErrorCode := -222; -- OpenCL initialization error
 OpenCLNoAMDBlasFft       : constant OpenCVErrorCode := -223;
 


 function  QCVQCVException_create return QCVExceptionH;
 pragma Import(C,QCVQCVException_create, "QCVQCVException_create");
 
 function  QCVQCVException_create2(code:integer;err: QStringH;func: QStringH;file: QStringH;line: integer) return QCVExceptionH;
 pragma Import(C,QCVQCVException_create2, "QCVQCVException_create2");
 
 function  QCVQCVException_what(handle:not null QCVExceptionH) return QStringH;
 pragma Import(C,QCVQCVException_what, "QCVQCVException_what");
 
 procedure QCVQCVException_formatMessage(handle:not null QCVExceptionH);
 pragma Import(C,QCVQCVException_formatMessage, "QCVQCVException_formatMessage");
 
 function  QCVQCVException_msg(handle:not null QCVExceptionH) return QStringH;
 pragma Import(C,QCVQCVException_msg, "QCVQCVException_msg");
 
 function  QCVQCVException_code(handle:not null QCVExceptionH) return OpenCVErrorCode;
 pragma Import(C,QCVQCVException_code, "QCVQCVException_code");
 
 function  QCVQCVException_err(handle:not null QCVExceptionH) return QStringH;
 pragma Import(C,QCVQCVException_err, "QCVQCVException_err");
 
 function  QCVQCVException_func(handle:not null QCVExceptionH) return QStringH;
 pragma Import(C,QCVQCVException_func, "QCVQCVException_func");
 
 function  QCVQCVException_file(handle:not null QCVExceptionH) return QStringH;
 pragma Import(C,QCVQCVException_file, "QCVQCVException_file");
 
 function  QCVQCVException_line(handle:not null QCVExceptionH) return integer;
 pragma Import(C,QCVQCVException_line, "QCVQCVException_line");
 
end;
