--
-- ͻ
--        D E S I G N   E N G I N E E R I N G            DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         QtOpenCVAda - Ada to Qt6 and OpenCV interface              
--                                                                    
--         Author :  Leonid Dulman     2021                           
--  Copyright (C) 2021                                                
-- ͼ
--
--   This library is free software; you can redistribute it and/or modify  *
--   it under the terms of the GNU Library General Public License as       *
--   published by the Free Software Foundation; either version 3 of the    *
--   License, or (at your option) any later version.                       *

with Qt.QSize;                use Qt.QSize;
with Qt.OpenCV;               use Qt.OpenCV;
with Qt.QCVCore;              use Qt.QCVCore;
with Qt.QCVMat;               use Qt.QCVMat;
with Qt.QCVImgProc;           use Qt.QCVImgProc;
with Qt.QCVCascadeClassifier; use Qt.QCVCascadeClassifier;
with Interfaces.C;            use Interfaces.C;



package body Qt.QCVFaceDetector is

 function QCVFaceDetector_detectAndDraw(frame: QCVMatH; cascade,nestedCascade : QCVCascadeClassifierH;scale: qreal := 1.0) return boolean is
 faces           : QCVFacesH := QCVFaces_create;
 nestedObjects   : QCVFacesH := QCVFaces_create;
 fx              : qreal;
 gray            : QCVMatH   := QCVMat_create;
 smallImg        : QCVMatH   := QCVMat_create;
 smallImgRoi     : QCVMatH   := QCVMat_create;
 size,x,y        : integer;
 radius, reye    : integer;
 aspect_ratio    : qreal;
 r,nr            : QCVRectH  :=  QCVRect_create;
 center,ceye     : QCVPointH := QCVPoint_create;
 color           : QCVScalarH := QCVScalar_create(255.0, 0.0, 0.0); -- Color for Drawing tool
 colorf          : QCVScalarH := QCVScalar_create(0.0, 255.0, 0.0); -- Color for Drawing tool
 isFind          : boolean;
 begin
  QCVImgProc_cvtColor(frame,gray, COLOR_BGR2GRAY); -- Convert to Gray Scale
  fx := 1.0/scale;
   --  Resize the Grayscale Image
  QCVCore_resize(gray,smallImg,QSize_create,fx,fx,INTER_LINEAR);
  QCVImgProc_equalizeHist( smallImg, smallImg );
      
  -- Detect faces of different sizes using cascade classifier 
  isFind := false;
  QCVCascadeClassifier_detectMultiScale(cascade,smallImg, faces, 1.1,2, CASCADE_SCALE_IMAGE, QCVSize_create(30, 30) );
  --  Draw circles around the faces
  size := QCVFaces_size(faces);
  center := QCVPoint_create;
  for i in 0 .. size -1 loop
    isFind := true;
    QCVFaces_at(faces,r,i); 
    aspect_ratio := qreal(QCVRect_width(r))/qreal(QCVRect_height(r));
    if aspect_ratio in 0.75 .. 1.3
    then
      radius := QCVImgProc_radiusFaceCircle(r,scale,center);
      QCVImgProc_circle(frame, center, radius, colorf, 3, 8, 0 );
    else
      QCVImgProc_rectangle(frame, 
      QCVPoint_create(integer(qreal(QCVRect_x(r))*scale), integer(qreal(QCVRect_y(r))*scale)),
      QCVPoint_create(integer(qreal(QCVRect_x(r) + QCVRect_width(r) -1 )*scale), integer(qreal(QCVRect_y(r) + QCVRect_height(r) - 1)*scale)),
      colorf, 3, 8, 0);
      if QCVCascadeClassifier_empty(nestedCascade) then goto endloop ; end if;
    end if;
    QCVMat_clone(smallImg,smallImgRoi);
        --  Detection of eyes int the input image
    QCVCascadeClassifier_detectMultiScale(nestedCascade, smallImgROI, nestedObjects, 1.1, 2,
                   CASCADE_SCALE_IMAGE, QCVSize_create(30, 30) );
    -- Draw circles around eyes
    for j in 0 .. QCVFaces_size(nestedObjects) -1 loop

      QCVFaces_at(nestedObjects,nr,j);
      reye := QCVImgProc_radiusFaceCircle(nr,scale,ceye);             
      QCVImgProc_circle(frame, ceye, reye, color, 3, 8, 0 );
    end loop;

  <<endloop>> 
  end loop;
  return isFind;
 end;
end;
