--
-- ͻ
--        D E S I G N   E N G I N E E R I N G            DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         QtOpenCVAda - Ada to Qt6 and OpenCV interface              
--                                                                    
--         Author :  Leonid Dulman     2021,2022                      
--  Copyright (C) 2012,2022                                           
-- ͼ
--
--   This library is free software; you can redistribute it and/or modify  *
--   it under the terms of the GNU Library General Public License as       *
--   published by the Free Software Foundation; either version 3 of the    *
--   License, or (at your option) any later version.                       *

with Qt.OpenCV;  use Qt.OpenCV;
with Qt.QString; use Qt.QString;
package Qt.QCVFaceDetector is

 type QCVFaceDetectorH is new QObjectH;

 -- There is the Fisherfaces algorithm used here
 
 type detectionTypes is (csv,frontal);
 pragma convention(C,detectionTypes);
 
 -- For frontal detection cascadeFileName is "Cascades/haarcascade_eye_tree_eyeglasses.xml"
 --                       csvFileName     is "Cascades/haarcascade_frontalcatface.xml"
 -- For csv     detection cascadeFileName is "Cascades/haarcascade_frontalface_default.xml"
 --                       csvFileName     is "facestemplates.txt" - manualy built                         
 
 function  QCVFaceDetector_create(facemark : not null QCVFacemarkH;cascadeFileName,csvFileName:QStringH;err: out integer;dt: detectionTypes := csv) return QCVFaceDetectorH;
 pragma Import(C,QCVFaceDetector_create, "QCVFaceDetector_create");
 
 procedure QCVFaceDetector_destroy(handle:not null QCVFaceDetectorH);
 pragma Import(C,QCVFaceDetector_destroy, "QCVFaceDetector_destroy");
 
 function  QCVFaceDetector_getErrorMessage(handle:not null QCVFaceDetectorH) return QStringH;
 pragma Import(C,QCVFaceDetector_getErrorMessage, "QCVFaceDetector_getErrorMessage");
 
 function  QCVFaceDetector_FindFaces(handle:not null QCVFaceDetectorH;gray:QCVMatH) return integer;
 pragma Import(C,QCVFaceDetector_FindFaces, "QCVFaceDetector_FindFaces");
 
 function  QCVFaceDetector_faceAt(handle:not null QCVFaceDetectorH;i:integer) return QCVRectH;
 pragma Import(C,QCVFaceDetector_faceAt, "QCVFaceDetector_faceAt");
 
 procedure QCVFaceDetector_drawFaceBox(handle:not null QCVFaceDetectorH;frame: QCVMatH;faceNumber:integer);
 pragma Import(C,QCVFaceDetector_drawFaceBox, "QCVFaceDetector_drawFaceBox");
 
 procedure QCVFaceDetector_detectMultiScale(handle:not null QCVFaceDetectorH;gr:QCVMatH);
 pragma Import(C,QCVFaceDetector_detectMultiScale, "QCVFaceDetector_detectMultiScale");
 
 function  QCVFaceDetector_getPrediction(handle:not null QCVFaceDetectorH;facemark: not null QCVFacemarkH;frame:not null QCVMatH;i:integer) return integer;
 pragma Import(C,QCVFaceDetector_getPrediction, "QCVFaceDetector_getPrediction");
 
 procedure QCVFaceDetector_putPredictionText(handle:not null QCVFaceDetectorH;original:QCVMatH;box_text:QStringH);
 pragma Import(C,QCVFaceDetector_putPredictionText, "QCVFaceDetector_putPredictionText");
 
 function  QCVFaceDetector_numberOfImagesForTrain(handle:not null QCVFaceDetectorH) return integer;
 pragma Import(C,QCVFaceDetector_numberOfImagesForTrain, "QCVFaceDetector_numberOfImagesForTrain");
 
 function  QCVFaceDetector_imageForTrainAt(handle:not null QCVFaceDetectorH;index:integer) return QCVMatH;
 pragma Import(C,QCVFaceDetector_imageForTrainAt, "QCVFaceDetector_imageForTrainAt");

 function  QCVFaceDetector_baseWidth(handle:not null QCVFaceDetectorH) return integer;
 pragma Import(C,QCVFaceDetector_baseWidth, "QCVFaceDetector_baseWidth");
 
 function  QCVFaceDetector_baseHeight(handle:not null QCVFaceDetectorH) return integer;
 pragma Import(C,QCVFaceDetector_baseHeight, "QCVFaceDetector_baseHeight");
 
 
 --if (framework == "caffe")
 --       net = cv::dnn::readNetFromCaffe(caffeConfigFile, caffeWeightFile);
 --   else
 --       net = cv::dnn::readNetFromTensorflow(tensorflowWeightFile, tensorflowConfigFile);
 
 --   if (device == "CPU")
 --   {
 --       net.setPreferableBackend(DNN_TARGET_CPU);
 --   }
 --   else
 --   {
 --       net.setPreferableBackend(DNN_BACKEND_CUDA);
 --       net.setPreferableTarget(DNN_TARGET_CUDA);
 --   } 
 
 procedure  QCVFaceDetector_detectFaceOpenCVDNN(frameOpenCVDNN:QCVMatH ;framework : QStringH := s2qs("caffe");device:QStringH := s2qs("CPU");caffeConfigFile :QStringH := s2qs("models/deploy.prototxt");caffeWeightFile:QStringH := s2qs("models/res10_300x300_ssd_iter_140000_fp16.caffemodel");tensorflowConfigFile :QStringH := s2qs("models/opencv_face_detector.pbtxt");tensorflowWeightFile :QStringH := s2qs("models/opencv_face_detector_uint8.pb");inWidth :qint64 :=  300;inHeight :qint64 :=  300;inScaleFactor :double := 1.0;confidenceThreshold :float := 0.7);
 pragma Import(C,QCVFaceDetector_detectFaceOpenCVDNN, "QCVFaceDetector_detectFaceOpenCVDNN");
 

 
 -- for year detector --------------------------------------------------------------------------- 
 function  QCVFaceDetector_detectAndDraw(handle:not null QCVFaceDetectorH;frame: QCVMatH;scale:qreal) return boolean;
 pragma Import(C,QCVFaceDetector_detectAndDraw, "QCVFaceDetector_detectAndDraw");
 
 function QCVFaceDetector_detectAndDraw(frame: QCVMatH; cascade,nestedCascade : QCVCascadeClassifierH;scale: qreal := 1.0) return boolean;
--------------------------------------------------- for people detection -----------------------------------
 function  QCVDNNDetector_detectPeople(imageIn: not null QCVMatH;model: not null QCVNetH;class_names:not null QStringListH) return QCVMatH;
 pragma Import(C,QCVDNNDetector_detectPeople, "QCVDNNDetector_detectPeople");
 
end;
