--
-- ͻ
--        D E S I G N   E N G I N E E R I N G            DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         QtOpenCVAda - Ada to Qt6 and OpenCV interface              
--                                                                    
--         Author :  Leonid Dulman     2021                           
--  Copyright (C) 2021                                                
-- ͼ
--
--   This library is free software; you can redistribute it and/or modify  *
--   it under the terms of the GNU Library General Public License as       *
--   published by the Free Software Foundation; either version 3 of the    *
--   License, or (at your option) any later version.                       *

with Qt.OpenCV; use Qt.OpenCV;
with builtin; use builtin;

package Qt.QCVHighGui is

 type GuiReceiverH     is new QObjectH;
 type CvWindowH        is new QWidgetH;
 type DefaultViewPortH is new QGraphicsViewH;
 type CvRectH          is new QRectH;
 type CvButtonBarH     is new QMenuBarH;
  
    CV_WINDOW_NORMAL       : constant := 0;        -- the user can resize the window (no constraint)  / also use to switch a fullscreen window to a normal size
    CV_WINDOW_AUTOSIZE     : constant := 1;        -- the user cannot resize the window, the size is constrainted by the image displayed
    CV_WINDOW_OPENGL       : constant := 16#1000#; -- window with opengl support

    -- Those flags are only for Qt
    CV_GUI_EXPANDED        : constant := 0;      --status bar and tool bar
    CV_GUI_NORMAL          : constant := 16#10#; -- old fashious way

    -- These 3 flags are used by cvNamedWindow and cvSet/GetWindowProperty
    CV_WINDOW_FULLSCREEN   : constant := 1;       -- change the window to fullscreen
    CV_WINDOW_FREERATIO    : constant := 16#100#; -- the image expends as much as it can (no ratio constraint)
    CV_WINDOW_KEEPRATIO    : constant := 0;       -- the ration image is respected.
    

 function  QCvWindow_create(title:QStringH;flag : integer :=  CV_WINDOW_NORMAL) return CvWindowH;
 pragma Import(C,QCvWindow_create, "QCvWindow_create");
 
 procedure QCvWindow_destroy(handle:not null CvWindowH);
 pragma Import(C,QCvWindow_destroy, "QCvWindow_destroy");
 
 procedure QCvWindow_setMouseCallBack(handle:not null CvWindowH;m:voidCallbackH;param: void_Star);
 pragma Import(C,QCvWindow_setMouseCallBack, "QCvWindow_setMouseCallBack");
 
 procedure QCvWindow_writeSettings(handle:not null CvWindowH);
 pragma Import(C,QCvWindow_writeSettings, "QCvWindow_writeSettings");
 
 procedure QCvWindow_readSettings(handle:not null CvWindowH);
 pragma Import(C,QCvWindow_readSettings, "QCvWindow_readSettings");
 
 function  QCvWindow_getRatio(handle:not null CvWindowH) return qreal;
 pragma Import(C,QCvWindow_getRatio, "QCvWindow_getRatio");
 
 procedure QCvWindow_setRatio(handle:not null CvWindowH;flags:integer);
 pragma Import(C,QCvWindow_setRatio, "QCvWindow_setRatio");
 
 function  QCvWindow_getWindowRect(handle:not null CvWindowH) return CvRectH;
 pragma Import(C,QCvWindow_getWindowRect, "QCvWindow_getWindowRect");
 
 function  QCvWindow_getPropWindow(handle:not null CvWindowH) return integer;
 pragma Import(C,QCvWindow_getPropWindow, "QCvWindow_getPropWindow");
 
 procedure QCvWindow_setPropWindow(handle:not null CvWindowH;flags:integer);
 pragma Import(C,QCvWindow_setPropWindow, "QCvWindow_setPropWindow");
 
 procedure QCvWindow_toggleFullScreen(handle:not null CvWindowH;flags:integer);
 pragma Import(C,QCvWindow_toggleFullScreen, "QCvWindow_toggleFullScreen");
 
 procedure QCvWindow_updateImage(handle:not null CvWindowH;arr:QCVMatH);
 pragma Import(C,QCvWindow_updateImage, "QCvWindow_updateImage");
 
 procedure QCvWindow_displayInfo(handle:not null CvWindowH;text:QStringH;delayms: integer);
 pragma Import(C,QCvWindow_displayInfo, "QCvWindow_displayInfo");
 
 procedure QCvWindow_displayStatusBar(handle:not null CvWindowH;text:QStringH;delayms: integer);
 pragma Import(C,QCvWindow_displayStatusBar, "QCvWindow_displayStatusBar");
 
 procedure QCvWindow_enablePropertiesButton(handle:not null CvWindowH);
 pragma Import(C,QCvWindow_enablePropertiesButton, "QCvWindow_enablePropertiesButton");
 
 procedure QCvWindow_setOpenGLDrawCallback(handle:not null CvWindowH;callback:voidCallbackH;userdata: void_Star);
 pragma Import(C,QCvWindow_setOpenGLDrawCallback, "QCvWindow_setOpenGLDrawCallback");
 
 procedure QCvWindow_makeCurrentOpenGLContext(handle:not null CvWindowH);
 pragma Import(C,QCvWindow_makeCurrentOpenGLContext, "QCvWindow_makeCurrentOpenGLContext");
 
 procedure QCvWindow_updateGL(handle:not null CvWindowH);
 pragma Import(C,QCvWindow_updateGL, "QCvWindow_updateGL");
 
 function  QCvWindow_isOpenGL(handle:not null CvWindowH) return boolean;
 pragma Import(C,QCvWindow_isOpenGL, "QCvWindow_isOpenGL");
 
 procedure QCvWindow_setViewportSize(handle:not null CvWindowH;size:QSizeH);
 pragma Import(C,QCvWindow_setViewportSize, "QCvWindow_setViewportSize");
 
 function  QCvWindow_param_flags(handle:not null CvWindowH) return integer;
 pragma Import(C,QCvWindow_param_flags, "QCvWindow_param_flags");
 
 function  QCvWindow_param_gui_mode(handle:not null CvWindowH) return integer;
 pragma Import(C,QCvWindow_param_gui_mode, "QCvWindow_param_gui_mode");
 
 function  QCvWindow_param_ratio_mode(handle:not null CvWindowH) return integer;
 pragma Import(C,QCvWindow_param_ratio_mode, "QCvWindow_param_ratio_mode");
 
 function  QCvWindow_myGlobalLayout(handle:not null CvWindowH) return QBoxLayoutH;
 pragma Import(C,QCvWindow_myGlobalLayout, "QCvWindow_myGlobalLayout");
 
 function  QCvWindow_myBarLayout(handle:not null CvWindowH) return QBoxLayoutH;
 pragma Import(C,QCvWindow_myBarLayout, "QCvWindow_myBarLayout");
 
 function  QCvWindow_vect_QActions(handle:not null CvWindowH) return QVectorH;
 pragma Import(C,QCvWindow_vect_QActions, "QCvWindow_vect_QActions");
 
 function  QCvWindow_myStatusBar(handle:not null CvWindowH) return QStatusBarH;
 pragma Import(C,QCvWindow_myStatusBar, "QCvWindow_myStatusBar");
 
 function  QCvWindow_myToolBar(handle:not null CvWindowH) return QToolBarH;
 pragma Import(C,QCvWindow_myToolBar, "QCvWindow_myToolBar");
 
 function  QCvWindow_myStatusBar_msg(handle:not null CvWindowH) return QLabelH;
 pragma Import(C,QCvWindow_myStatusBar_msg, "QCvWindow_myStatusBar_msg");
 
 function  QCvWindow_createButtonBar(handle:not null CvWindowH;bar_name:QStringH) return CvButtonbarH;
 pragma Import(C,QCvWindow_createButtonBar, "QCvWindow_createButtonBar");
 
 procedure QCvWindow_addSlider(handle:CvWindowH;name: QStringH;value: in out integer;count: integer;on_change : intCallbackH);
 pragma Import(C,QCvWindow_addSlider, "QCvWindow_addSlider");
 
 procedure QCvWindow_addSlider2(handle:CvWindowH;name: QStringH;value: in out integer;count: integer;on_change : intCallbackH; userdata : void_Star);
 pragma Import(C,QCvWindow_addSlider2, "QCvWindow_addSlider2");


end;
