--
-- ͻ
--        D E S I G N   E N G I N E E R I N G            DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         QtOpenCVAda - Ada to Qt6 and OpenCV interface              
--                                                                    
--         Author :  Leonid Dulman     2021,2022                      
--  Copyright (C) 2012,2022                                           
-- ͼ
--
--   This library is free software; you can redistribute it and/or modify  *
--   it under the terms of the GNU Library General Public License as       *
--   published by the Free Software Foundation; either version 3 of the    *
--   License, or (at your option) any later version.                       *

with Qt.QString;use Qt.QString;
with Qt.OpenCV; use Qt.OpenCV;
with Qt.QCVCore;use Qt.QCVCore;
package Qt.QCVImgProc is

 type MarkerTypes  is new integer range 0 .. 6;

 MARKER_CROSS        : constant MarkerTypes := 0; --   A crosshair marker shape
 MARKER_TILTED_CROSS : constant MarkerTypes := 1; --   A 45 degree tilted crosshair marker shape
 MARKER_STAR         : constant MarkerTypes := 2; --   A star marker shape, combination of cross and tilted cross
 MARKER_DIAMOND      : constant MarkerTypes := 3; --   A diamond marker shape
 MARKER_SQUARE       : constant MarkerTypes := 4; --   A square marker shape
 MARKER_TRIANGLE_UP  : constant MarkerTypes := 5; --   An upwards pointing triangle marker shape
 MARKER_TRIANGLE_DOWN: constant MarkerTypes := 6; --   A downwards pointing triangle marker shape

 procedure QCVImgProc_cvtColor(src,dst: QCVMatH;code: ColorConversionCodes;dstCn : integer :=  0 );
 pragma Import(C,QCVImgProc_cvtColor, "QCVImgProc_cvtColor");
 
 procedure QCVImgProc_equalizeHist(src,dst:QCVMatH);
 pragma Import(C,QCVImgProc_equalizeHist, "QCVImgProc_equalizeHist");
 
 procedure QCVImgProc_rectangle(frame:QCVMatH;pt1,pt2: QCVPointH;color:QCVScalarH;thickness : integer :=  1;lineType : LineTypes :=  LINE_8;shift : integer :=  0);
 pragma Import(C,QCVImgProc_rectangle, "QCVImgProc_rectangle");
 
 procedure QCVImgProc_rectangle2(frame:QCVMatH;rect:QCVRectH;color:QCVScalarH;thickness : integer :=  1;lineType : LineTypes :=  LINE_8;shift : integer :=  0);
 pragma Import(C,QCVImgProc_rectangle2, "QCVImgProc_rectangle2");
 
 procedure QCVImgProc_circle(frame:QCVMatH;center: QCVPointH;radius: int;color:QCVScalarH;thickness : integer :=  1;lineType : LineTypes :=  LINE_8;shift : integer :=  0);
 pragma Import(C,QCVImgProc_circle, "QCVImgProc_circle");
 
 function  QCVImgProc_radiusFaceCircle(face:QCVRectH;scale:qreal;center:QCVPointH) return integer;
 pragma Import(C,QCVImgProc_radiusFaceCircle, "QCVImgProc_radiusFaceCircle");

 function  QCVImgProc_minEnclosingTriangle(points,triangle:not null QCVMatH) return qreal;
 pragma Import(C,QCVImgProc_minEnclosingTriangle, "QCVImgProc_minEnclosingTriangle");
 
 procedure QCVImgProc_cvtColorTwoPlane(src1,src2,dst:not null QCVMatH;code : integer);
 pragma Import(C,QCVImgProc_cvtColorTwoPlane, "QCVImgProc_cvtColorTwoPlane");
 
 procedure QCVImgProc_ellipse(img: not null QCVMatH;center: not null QCVPointH;axes: not null QCVSizeH;angle,startAngle,endAngle: qreal;color:not null QCVScalarH;thickness : integer :=  1;lineType :LineTypes :=  LINE_8;shift : integer :=  0);
 pragma Import(C,QCVImgProc_ellipse, "QCVImgProc_ellipse");
 
 procedure QCVImgProc_drawMarker(img:not null QCVMatH;position: not null QCVPointH;color: not null QCVScalarH;markerType :MarkerTypes :=  MARKER_CROSS;markerSize: integer := 20;thickness: integer := 1;lineType:LineTypes :=  LINE_8);
 pragma Import(C,QCVImgProc_drawMarker, "QCVImgProc_drawMarker");
 
 function  QCVImgProc_imread(fileName: QStringH) return QCVMatH;
 pragma Import(C,QCVImgProc_imread, "QCVImgProc_imread");

 function  QCVImgProc_imread2(fileName: QStringH;mode : Qt.OpenCV.ImreadModes) return QCVMatH;
 pragma Import(C,QCVImgProc_imread2, "QCVImgProc_imread2");

 procedure QCVImgProc_imwrite(fileName: QStringH;frame:QCVMatH);
 pragma Import(C,QCVImgProc_imwrite, "QCVImgProc_imwrite");
 
 ---------------------------------------------------------------------------------------------------------
 type CPU_type is (cpu,gpu);
 pragma convention(C,CPU_type);
 
 -- fixed input size for the pre-trained network 224x224

 procedure QCVImgProc_ImageColorize(frame,colorFrame : not null QCVMatH; cp : CPU_type :=  cpu;protoFile :QStringH := s2qs("./models/colorization_deploy_v2.prototxt");weightsFile : QStringH := s2qs("./models/colorization_release_v2.caffemodel");W_in : integer :=  224;H_in : integer :=  224 );
 pragma Import(C,QCVImgProc_ImageColorize, "QCVImgProc_ImageColorize");
 
 function  QCVImgProc_VideoColorize(videoFileName : not null QStringH;colorizedVideoFileName : not null QStringH := s2qs("colorizedVideo.avi"); cp : CPU_type :=  cpu;protoFile :QStringH := s2qs("./models/colorization_deploy_v2.prototxt");weightsFile : QStringH := s2qs("./models/colorization_release_v2.caffemodel");W_in : integer :=  224;H_in : integer :=  224 ) return boolean;
 pragma Import(C,QCVImgProc_VideoColorize, "QCVImgProc_VideoColorize");
 
 procedure QCVImgProc_handPoseImage(frame : not null QCVMatH;frameOut : not null QCVMatH; protoFile :QStringH := s2qs("model/hand/pose_deploy.prototxt"); weightsFile :QStringH := s2qs("model/hand/pose_iter_102000.caffemodel"); thresh :float :=  0.01;inHeight :integer := 368;nPoints :integer := 22);
 pragma Import(C,QCVImgProc_handPoseImage, "QCVImgProc_handPoseImage");
 
 function  QCVImgProc_handPoseVideo(cap : not null QCVVideoCaptureH;markedVideo :not null QStringH :=  s2qs("ColorizedVideo.avi");protoFile : QStringH := s2qs("hand/pose_deploy.prototxt"); weightsFile : QStringH := s2qs("hand/pose_iter_102000.caffemodel");thresh : float :=  0.01;inHeight :integer := 368; nPoints :integer :=  22) return boolean;
 pragma Import(C,QCVImgProc_handPoseVideo, "QCVImgProc_handPoseVideo");

 function  QCVImgProc_removeRedEyes(frame,frameOut: not null QCVMatH;haar_cascade_model :QStringH :=  s2qs("model/haarcascade_eye.xml")) return boolean;
 pragma Import(C,QCVImgProc_removeRedEyes, "QCVImgProc_removeRedEyes");
 
 function  QCVImgProc_openPoseImage(frame,frameCopy: not null QCVMatH;cp :CPU_type :=  cpu;protoFile : QStringH := s2qs("model/pose/mpi/pose_deploy_linevec_faster_4_stages.prototxt"); weightsFile : QStringH :=  s2qs("model/pose/mpi/pose_iter_160000.caffemodel"); inWidth: integer :=  368;inHeight : integer :=  368;thresh : float :=  0.1;nPoints :integer :=  15) return boolean;
 pragma Import(C,QCVImgProc_openPoseImage, "QCVImgProc_openPoseImage");
 
 function  QCVImgProc_openPoseVideo(cap : not null QCVVideoCaptureH;markedVideo :not null QStringH :=  s2qs("MarkedVideo.avi");cp :CPU_type :=  cpu;protoFile : QStringH := s2qs("pose/mpi/pose_deploy_linevec_faster_4_stages.prototxt"); weightsFile : QStringH :=  s2qs("pose/mpi/pose_iter_160000.caffemodel"); inWidth: integer :=  368;inHeight : integer :=  368;thresh : float :=  0.1;nPoints :integer :=  15) return boolean;
 pragma Import(C,QCVImgProc_openPoseVideo, "QCVImgProc_openPoseVideo");
   
 function  QCVImgProc_textDetect(cap: not null QCVVideoCaptureH;model :QStringH :=  s2qs("model/frozen_east_text_detection.pb"); cp : CPU_type :=  cpu;inpWidth :integer :=  320;inpHeight : integer :=  320;confThreshold : float :=  0.5;nmsThreshold : float :=  0.4) return boolean;
 pragma Import(C,QCVImgProc_textDetect, "QCVImgProc_textDetect");
 
 function  QCVImgProc_facialLandmarkDetection(cam: not null QCVVideoCaptureH;faceModel :QStringH :=  s2qs("model/lbfmodel.yaml");frontalFace :QStringH :=  s2qs("model/haarcascade_frontalface_alt2.xml")) return boolean;
 pragma Import(C,QCVImgProc_facialLandmarkDetection, "QCVImgProc_facialLandmarkDetection");

 function  QCVImgProc_ninjaEyeDetector(frame,frameOut : not null QCVMatH; frontalfacemodel:QStringH := s2qs("model/cascade/mallick_haarcascade_frontalface_default.xml"); min_face_size : integer :=  100;max_face_size :integer :=  300) return boolean;
 pragma Import(C,QCVImgProc_ninjaEyeDetector, "QCVImgProc_ninjaEyeDetector");
 
 procedure QCVImgProc_qrCodeImageDecoder(frame:not null QCVMatH;list: not null QStringListH;detector_prototxt :QStringH := s2qs("model/detect.prototxt"); detector_caffe_model : QStringH := s2qs("model/detect.caffemodel"); super_resolution_prototxt : QStringH :=  s2qs("model/sr.prototxt"); super_resolution_caffe_model : QStringH := s2qs("model/sr.caffemodel"));
 pragma Import(C,QCVImgProc_qrCodeImageDecoder, "QCVImgProc_qrCodeImageDecoder");
  
 function  QCVImgProc_ageGender(frame,frameFace: not null QCVMatH; cp:CPU_type := cpu; faceProto:QStringH := s2qs("model/opencv_face_detector.pbtxt");faceModel: QStringH := s2qs("model/opencv_face_detector_uint8.pb"); ageProto: QStringH := s2qs("model/age_deploy.prototxt"); ageModel: QStringH := s2qs("model/age_net.caffemodel");genderProto : QStringH := s2qs("model/gender_deploy.prototxt"); genderModel : QStringH := s2qs("model/gender_net.caffemodel")) return boolean;
 pragma Import(C,QCVImgProc_ageGender, "QCVImgProc_ageGender");
 
 procedure QCVImgProc_textRecognizer(frame: not null QCVMatH;list: not null QStringListH; detModelPath :QStringH := s2qs("model/frozen_east_text_detection.pb"); recModelPath: QStringH := s2qs("model/ResNet_CTC.onnx"); vocPath: QStringH := s2qs("model/alphabet_36.txt"); width: integer := 320; height: integer := 320; confThreshold : float := 0.5;nmsThreshold : float := 0.4; imreadRGB:integer := 0);
 pragma Import(C,QCVImgProc_textRecognizer, "QCVImgProc_textRecognizer");
 
 procedure QCVImgProc_barcodeDetector(handle:not null QCVMatH;codes:not null QStringListH);
 pragma Import(C,QCVImgProc_barcodeDetector, "QCVImgProc_barcodeDetector");
 
-------------------------------------------------------------------
 procedure QCVImgProc_rectangle(frame:QCVMatH;rect:QCVRectH;color:QCVScalarH;thickness : integer :=  1;lineType : LineTypes :=  LINE_8;shift : integer :=  0);
 function  QCVImgProc_imread(fileName: QStringH;mode : Qt.OpenCV.ImreadModes) return QCVMatH;
 
end;
