--
-- ͻ
--        D E S I G N   E N G I N E E R I N G            DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         QtOpenCVAda - Ada to Qt6 and OpenCV interface              
--                                                                    
--         Author :  Leonid Dulman     2021,2022                      
--  Copyright (C) 2021,2022                                           
-- ͼ
--
--   This library is free software; you can redistribute it and/or modify  *
--   it under the terms of the GNU Library General Public License as       *
--   published by the Free Software Foundation; either version 3 of the    *
--   License, or (at your option) any later version.                       *

with Qt;         use Qt;
with Qt.OpenCV;  use Qt.OpenCV;
with Qt.QString; use Qt.QString;
package Qt.QCVVideoCapture is

 type VideoCaptureAPIs is new integer;
 CAP_ANY          : constant VideoCaptureAPIs := 0;            -- Auto detect == 0
 CAP_VFW          : constant VideoCaptureAPIs := 200;          -- Video For Windows (obsolete, removed)
 CAP_V4L          : constant VideoCaptureAPIs := 200;          -- V4L/V4L2 capturing support
 CAP_V4L2         : constant VideoCaptureAPIs := CAP_V4L;      -- Same as CAP_V4L
 CAP_FIREWIRE     : constant VideoCaptureAPIs := 300;          -- IEEE 1394 drivers
 CAP_FIREWARE     : constant VideoCaptureAPIs := CAP_FIREWIRE; -- Same value as CAP_FIREWIRE
 CAP_IEEE1394     : constant VideoCaptureAPIs := CAP_FIREWIRE; -- Same value as CAP_FIREWIRE
 CAP_DC1394       : constant VideoCaptureAPIs := CAP_FIREWIRE; -- Same value as CAP_FIREWIRE
 CAP_CMU1394      : constant VideoCaptureAPIs := CAP_FIREWIRE; -- Same value as CAP_FIREWIRE
 CAP_QT           : constant VideoCaptureAPIs := 500;          -- QuickTime (obsolete, removed)
 CAP_UNICAP       : constant VideoCaptureAPIs := 600;          -- Unicap drivers (obsolete, removed)
 CAP_DSHOW        : constant VideoCaptureAPIs := 700;          -- DirectShow (via videoInput)
 CAP_PVAPI        : constant VideoCaptureAPIs := 800;          -- PvAPI, Prosilica GigE SDK
 CAP_OPENNI       : constant VideoCaptureAPIs := 900;          -- OpenNI (for Kinect)
 CAP_OPENNI_ASUS  : constant VideoCaptureAPIs := 910;          -- OpenNI (for Asus Xtion)
 CAP_ANDROID      : constant VideoCaptureAPIs := 1000;         -- Android - not used
 CAP_XIAPI        : constant VideoCaptureAPIs := 1100;         -- XIMEA Camera API
 CAP_AVFOUNDATION : constant VideoCaptureAPIs := 1200;         -- AVFoundation framework for iOS (OS X Lion will have the same API)
 CAP_GIGANETIX    : constant VideoCaptureAPIs := 1300;         -- Smartek Giganetix GigEVisionSDK
 CAP_MSMF         : constant VideoCaptureAPIs := 1400;         -- Microsoft Media Foundation (via videoInput)
 CAP_WINRT        : constant VideoCaptureAPIs := 1410;         -- Microsoft Windows Runtime using Media Foundation
 CAP_INTELPERC    : constant VideoCaptureAPIs := 1500;         -- RealSense (former Intel Perceptual Computing SDK)
 CAP_REALSENSE    : constant VideoCaptureAPIs := 1500;         -- Synonym for CAP_INTELPERC
 CAP_OPENNI2      : constant VideoCaptureAPIs := 1600;         -- OpenNI2 (for Kinect)
 CAP_OPENNI2_ASUS : constant VideoCaptureAPIs := 1610;         -- OpenNI2 (for Asus Xtion and Occipital Structure sensors)
 CAP_GPHOTO2      : constant VideoCaptureAPIs := 1700;         -- gPhoto2 connection
 CAP_GSTREAMER    : constant VideoCaptureAPIs := 1800;         -- GStreamer
 CAP_FFMPEG       : constant VideoCaptureAPIs := 1900;         -- Open and record video file or stream using the FFMPEG library
 CAP_IMAGES       : constant VideoCaptureAPIs := 2000;         -- OpenCV Image Sequence (e.g. img_%02d.jpg)
 CAP_ARAVIS       : constant VideoCaptureAPIs := 2100;         -- Aravis SDK
 CAP_OPENCV_MJPEG : constant VideoCaptureAPIs := 2200;         -- Built-in OpenCV MotionJPEG codec
 CAP_INTEL_MFX    : constant VideoCaptureAPIs := 2300;         -- Intel MediaSDK
 CAP_XINE         : constant VideoCaptureAPIs := 2400;         -- XINE engine (Linux)
 CAP_UEYE         : constant VideoCaptureAPIs := 2500;         --   uEye Camera API 
  
 type VideoCaptureProperties  is  new integer range 0 .. 68;
 CAP_PROP_POS_MSEC                   : constant VideoCaptureProperties := 0;-- Current position of the video file in milliseconds.
 CAP_PROP_POS_FRAMES                 : constant VideoCaptureProperties := 1;-- 0-based index of the frame to be decoded/captured next.
 CAP_PROP_POS_AVI_RATIO              : constant VideoCaptureProperties := 2;-- Relative position of the video file: 0=start of the film; 1=end of the film.
 CAP_PROP_FRAME_WIDTH                : constant VideoCaptureProperties := 3;-- Width of the frames in the video stream.
 CAP_PROP_FRAME_HEIGHT               : constant VideoCaptureProperties := 4;-- Height of the frames in the video stream.
 CAP_PROP_FPS                        : constant VideoCaptureProperties :=5 ;-- Frame rate.
 CAP_PROP_FOURCC                     : constant VideoCaptureProperties :=6 ;-- 4-character code of codec. see VideoWriter::fourcc .
 CAP_PROP_FRAME_COUNT                : constant VideoCaptureProperties :=7 ;-- Number of frames in the video file.
 CAP_PROP_FORMAT                     : constant VideoCaptureProperties :=8 ;-- Format of the %Mat objects returned by VideoCapture::retrieve().
 CAP_PROP_MODE                       : constant VideoCaptureProperties :=9 ;-- Backend-specific value indicating the current capture mode.
 CAP_PROP_BRIGHTNESS                 : constant VideoCaptureProperties :=10; -- Brightness of the image (only for those cameras that support).
 CAP_PROP_CONTRAST                   : constant VideoCaptureProperties :=11; -- Contrast of the image (only for cameras).
 CAP_PROP_SATURATION                 : constant VideoCaptureProperties :=12; -- Saturation of the image (only for cameras).
 CAP_PROP_HUE                        : constant VideoCaptureProperties :=13; -- Hue of the image (only for cameras).
 CAP_PROP_GAIN                       : constant VideoCaptureProperties :=14; -- Gain of the image (only for those cameras that support).
 CAP_PROP_EXPOSURE                   : constant VideoCaptureProperties :=15; -- Exposure (only for those cameras that support).
 CAP_PROP_CONVERT_RGB                : constant VideoCaptureProperties :=16; -- Boolean flags indicating whether images should be converted to RGB.
 CAP_PROP_WHITE_BALANCE_BLUE_U       : constant VideoCaptureProperties :=17; -- Currently unsupported.
 CAP_PROP_RECTIFICATION              : constant VideoCaptureProperties :=18; -- Rectification flag for stereo cameras (note: only supported by DC1394 v 2.x backend currently).
 CAP_PROP_MONOCHROME                 : constant VideoCaptureProperties :=19;
 CAP_PROP_SHARPNESS                  : constant VideoCaptureProperties :=20;
 CAP_PROP_AUTO_EXPOSURE              : constant VideoCaptureProperties :=21; -- DC1394: exposure control done by camera; user can adjust reference level using this feature.
 CAP_PROP_GAMMA                      : constant VideoCaptureProperties :=22;
 CAP_PROP_TEMPERATURE                : constant VideoCaptureProperties :=23;
 CAP_PROP_TRIGGER                    : constant VideoCaptureProperties :=24;
 CAP_PROP_TRIGGER_DELAY              : constant VideoCaptureProperties :=25;
 CAP_PROP_WHITE_BALANCE_RED_V        : constant VideoCaptureProperties :=26;
 CAP_PROP_ZOOM                       : constant VideoCaptureProperties :=27;
 CAP_PROP_FOCUS                      : constant VideoCaptureProperties :=28;
 CAP_PROP_GUID                       : constant VideoCaptureProperties :=29;
 CAP_PROP_ISO_SPEED                  : constant VideoCaptureProperties :=30;
 CAP_PROP_BACKLIGHT                  : constant VideoCaptureProperties :=32;
 CAP_PROP_PAN                        : constant VideoCaptureProperties :=33;
 CAP_PROP_TILT                       : constant VideoCaptureProperties :=34;
 CAP_PROP_ROLL                       : constant VideoCaptureProperties :=35;
 CAP_PROP_IRIS                       : constant VideoCaptureProperties :=36;
 CAP_PROP_SETTINGS                   : constant VideoCaptureProperties :=37; -- Pop up video/camera filter dialog (note: only supported by DSHOW backend currently. The property value is ignored)
 CAP_PROP_BUFFERSIZE                 : constant VideoCaptureProperties :=38;
 CAP_PROP_AUTOFOCUS                  : constant VideoCaptureProperties :=39;
 CAP_PROP_SAR_NUM                    : constant VideoCaptureProperties :=40; -- Sample aspect ratio: num/den (num)
 CAP_PROP_SAR_DEN                    : constant VideoCaptureProperties :=41; -- Sample aspect ratio: num/den (den)
 CAP_PROP_BACKEND                    : constant VideoCaptureProperties :=42; -- Current backend (enum VideoCaptureAPIs). Read-only property
 CAP_PROP_CHANNEL                    : constant VideoCaptureProperties :=43; -- Video input or Channel Number (only for those cameras that support)
 CAP_PROP_AUTO_WB                    : constant VideoCaptureProperties :=44; -- enable/ disable auto white-balance
 CAP_PROP_WB_TEMPERATURE             : constant VideoCaptureProperties :=45; -- white-balance color temperature 
 CAP_PROP_CODEC_PIXEL_FORMAT         : constant VideoCaptureProperties :=46; -- (read-only) codec's pixel format. 4-character code - see VideoWriter::fourcc . Subset of [AV_PIX_FMT_*](https://github.com/FFmpeg/FFmpeg/blob/master/libavcodec/raw.c) or -1 if unknow
 CAP_PROP_BITRATE                    : constant VideoCaptureProperties :=47; -- (read-only) Video bitrate in kbits/
 CAP_PROP_ORIENTATION_META           : constant VideoCaptureProperties :=48; -- (read-only) Frame rotation defined by stream meta (applicable for FFmpeg back-end only
 CAP_PROP_ORIENTATION_AUTO           : constant VideoCaptureProperties :=49; -- f true - rotates output frames of CvCapture considering video file's metadata  (applicable for FFmpeg back-end only) (https://github.com/opencv/opencv/issues/15499
 CAP_PROP_HW_ACCELERATION            : constant VideoCaptureProperties :=50; -- (**open-only**) Hardware acceleration type (see #VideoAccelerationType). Setting supported only via `params` parameter in cv::VideoCapture constructor / .open() method. Default value is backend-specific
 CAP_PROP_HW_DEVICE                  : constant VideoCaptureProperties :=51; -- (**open-only**) Hardware device index (select GPU if multiple available). Device enumeration is acceleration type specific
 CAP_PROP_HW_ACCELERATION_USE_OPENCL : constant VideoCaptureProperties :=52; -- (**open-only**) If non-zero, create new OpenCL context and bind it to current thread. The OpenCL context created with Video Acceleration context attached it (if not attached yet) for optimized GPU data copy between HW accelerated decoder and cv::UMat
 CAP_PROP_OPEN_TIMEOUT_MSEC          : constant VideoCaptureProperties :=53; -- (**open-only**) timeout in milliseconds for opening a video capture (applicable for FFmpeg back-end only
 CAP_PROP_READ_TIMEOUT_MSEC          : constant VideoCaptureProperties :=54; -- (**open-only**) timeout in milliseconds for reading from a video capture (applicable for FFmpeg back-end only
 CAP_PROP_STREAM_OPEN_TIME_USEC      : constant VideoCaptureProperties :=55; -- (read-only) time in microseconds since Jan 1 1970 when stream was opened. Applicable for FFmpeg backend only. Useful for RTSP and other live stream
 CAP_PROP_VIDEO_TOTAL_CHANNELS       : constant VideoCaptureProperties :=56; -- (read-only) Number of video channel
 CAP_PROP_VIDEO_STREAM               : constant VideoCaptureProperties :=57; -- (**open-only**) Specify video stream, 0-based index. Use -1 to disable video stream from file or IP cameras. Default value is 0
 CAP_PROP_AUDIO_STREAM               : constant VideoCaptureProperties :=58; -- (**open-only**) Specify stream in multi-language media files, -1 - disable audio processing or microphone. Default value is -1
 CAP_PROP_AUDIO_POS                  : constant VideoCaptureProperties :=59; -- (read-only) Audio position is measured in samples. Accurate audio sample timestamp of previous grabbed fragment. See CAP_PROP_AUDIO_SAMPLES_PER_SECOND and CAP_PROP_AUDIO_SHIFT_NSEC
 CAP_PROP_AUDIO_SHIFT_NSEC           : constant VideoCaptureProperties :=60; -- (read only) Contains the time difference between the start of the audio stream and the video stream in nanoseconds. Positive value means that audio is started after the first video frame. Negative value means that audio is started before the first video frame
 CAP_PROP_AUDIO_DATA_DEPTH           : constant VideoCaptureProperties :=61; -- (open, read) Alternative definition to bits-per-sample, but with clear handling of 32F / 32
 CAP_PROP_AUDIO_SAMPLES_PER_SECOND   : constant VideoCaptureProperties :=62; -- (open, read) determined from file/codec input. If not specified, then selected audio sample rate is 4410
 CAP_PROP_AUDIO_BASE_INDEX           : constant VideoCaptureProperties :=63; -- (read-only) Index of the first audio channel for .retrieve() calls. That audio channel number continues enumeration after video channels
 CAP_PROP_AUDIO_TOTAL_CHANNELS       : constant VideoCaptureProperties :=64; -- (read-only) Number of audio channels in the selected audio stream (mono, stereo, etc
 CAP_PROP_AUDIO_TOTAL_STREAMS        : constant VideoCaptureProperties :=65; -- (read-only) Number of audio streams
 CAP_PROP_AUDIO_SYNCHRONIZE          : constant VideoCaptureProperties :=66; -- (open, read) Enables audio synchronization
 CAP_PROP_LRF_HAS_KEY_FRAME          : constant VideoCaptureProperties :=67; -- FFmpeg back-end only - Indicates whether the Last Raw Frame (LRF), output from VideoCapture::read() when VideoCapture is initialized with VideoCapture::open(CAP_FFMPEG, {CAP_PROP_FORMAT, -1}) or VideoCapture::set(CAP_PROP_FORMAT,-1) is called before the first call to VideoCapture::read(), contains encoded data for a key frame
 CAP_PROP_CODEC_EXTRADATA_INDEX      : constant VideoCaptureProperties :=68; -- Positive index indicates that returning extra data is supported by the video back end.  This can be retrieved as cap.retrieve(data, <returned index>).  E.g. When reading from a h264 encoded RTSP stream, the FFmpeg backend could return the SPS and/or PPS if available (if sent in reply to a DESCRIBE request), from calls to cap.retrieve(data, <returned index>)
       
  type  VideoWriterProperties is new integer range 1 .. 8;
  VIDEOWRITER_PROP_QUALITY                    : constant VideoWriterProperties := 1;    -- Current quality (0..100%) of the encoded videostream. Can be adjusted dynamically in some codecs.
  VIDEOWRITER_PROP_FRAMEBYTES                 : constant VideoWriterProperties := 2; -- (Read-only): Size of just encoded video frame. Note that the encoding order may be different from representation order.
  VIDEOWRITER_PROP_NSTRIPES                   : constant VideoWriterProperties := 3;   -- Number of stripes for parallel encoding. -1 for auto detection.
  VIDEOWRITER_PROP_IS_COLOR                   : constant VideoWriterProperties := 4; -- If it is not zero, the encoder will expect and encode color frames, otherwise i
                                                                                     -- will work with grayscale frames
  VIDEOWRITER_PROP_DEPTH                      : constant VideoWriterProperties := 5; -- Defaults to CV_8U
  VIDEOWRITER_PROP_HW_ACCELERATION            : constant VideoWriterProperties := 6; -- (**open-only**) Hardware acceleration type (see #VideoAccelerationType). Setting supported only via `params` parameter in VideoWriter constructor / .open() method. Default value is backend-specific
  VIDEOWRITER_PROP_HW_DEVICE                  : constant VideoWriterProperties := 7; -- (**open-only**) Hardware device index (select GPU if multiple available). Device enumeration is acceleration type specific
  VIDEOWRITER_PROP_HW_ACCELERATION_USE_OPENCL : constant VideoWriterProperties := 8; -- (**open-only**) If non-zero, create new OpenCL context and bind it to current thread. The OpenCL context created with Video Acceleration context attached it (if not attached yet) for optimized GPU data copy between cv::UMat and HW accelerated encoder

  type VideoAccelerationType is (
    VIDEO_ACCELERATION_NONE,  -- Do not require any specific H/W acceleration, prefer software processing.
                              -- Reading of this value means that special H/W accelerated handling is not added or not detected by OpenCV.
    VIDEO_ACCELERATION_ANY,   -- Prefer to use H/W acceleration. If no one supported, then fallback to software processing.
                              -- @note H/W acceleration may require special configuration of used environment.
                              -- @note Results in encoding scenario may differ between software and hardware accelerated encoders.
    VIDEO_ACCELERATION_D3D11, -- DirectX 11
    VIDEO_ACCELERATION_VAAPI, -- VAAPI
    VIDEO_ACCELERATION_MFX     -- libmfx (Intel MediaSDK/oneVPL) 
  );
  pragma convention(C,VideoAccelerationType);

  CAP_PROP_DC1394_OFF                : constant := -4; -- turn the feature off (not controlled manually nor automatically).
  CAP_PROP_DC1394_MODE_MANUAL        : constant := -3; -- set automatically when a value of the feature is set by the user.
  CAP_PROP_DC1394_MODE_AUTO          : constant := -2;
  CAP_PROP_DC1394_MODE_ONE_PUSH_AUTO : constant := -1;
  CAP_PROP_DC1394_MAX                : constant := 31; 
       
 -- Properties of cameras available through OpenNI backend
  CAP_PROP_OPENNI_OUTPUT_MODE       : constant := 100;
  CAP_PROP_OPENNI_FRAME_MAX_DEPTH   : constant := 101; -- In mm
  CAP_PROP_OPENNI_BASELINE          : constant := 102; -- In mm
  CAP_PROP_OPENNI_FOCAL_LENGTH      : constant := 103; -- In pixels
  CAP_PROP_OPENNI_REGISTRATION      : constant := 104; -- Flag that synchronizes the remapping depth map to image map
                                                -- by changing depth generator's view point (if the flag is "on") or
                                                -- sets this view point to its normal one (if the flag is "off").
  CAP_PROP_OPENNI_REGISTRATION_ON   : constant := CAP_PROP_OPENNI_REGISTRATION;
  CAP_PROP_OPENNI_APPROX_FRAME_SYNC : constant := 105;
  CAP_PROP_OPENNI_MAX_BUFFER_SIZE   : constant := 106;
  CAP_PROP_OPENNI_CIRCLE_BUFFER     : constant := 107;
  CAP_PROP_OPENNI_MAX_TIME_DURATION : constant := 108;
  CAP_PROP_OPENNI_GENERATOR_PRESENT : constant := 109;
  CAP_PROP_OPENNI2_SYNC             : constant := 110;
  CAP_PROP_OPENNI2_MIRROR           : constant := 111;
     
  -- Supported output modes of OpenNI image generator
  CAP_OPENNI_VGA_30HZ               : constant := 0;
  CAP_OPENNI_SXGA_15HZ              : constant := 1;
  CAP_OPENNI_SXGA_30HZ              : constant := 2;
  CAP_OPENNI_QVGA_30HZ              : constant := 3;
  CAP_OPENNI_QVGA_60HZ              : constant := 4;

 function  QVideoCapture_create return  QCVVideoCaptureH;
 pragma Import(C,QVideoCapture_create, "QVideoCapture_create");
 
 function  QVideoCapture_create2(fileName: QCVStringH;apiPreference : VideoCaptureAPIs := CAP_ANY) return  QCVVideoCaptureH;
 pragma Import(C,QVideoCapture_create2, "QVideoCapture_create2");
 
 function  QVideoCapture_create3(index:integer;apiPreference : VideoCaptureAPIs := CAP_ANY) return  QCVVideoCaptureH;
 pragma Import(C,QVideoCapture_create3, "QVideoCapture_create3");
 
 procedure QVideoCapture_destroy(handle:not null  QCVVideoCaptureH);
 pragma Import(C,QVideoCapture_destroy, "QVideoCapture_destroy");
 
 function  QVideoCapture_open(handle:not null  QCVVideoCaptureH;fileName: QStringH;apiPreference : VideoCaptureAPIs := CAP_ANY) return  boolean;
 pragma Import(C,QVideoCapture_open, "QVideoCapture_open");
 
 function  QVideoCapture_open2(handle:not null  QCVVideoCaptureH;index:integer;apiPreference : VideoCaptureAPIs := CAP_ANY) return  boolean;
 pragma Import(C,QVideoCapture_open2, "QVideoCapture_open2");
 
 function  QVideoCapture_isOpened(handle:not null  QCVVideoCaptureH) return  boolean;
 pragma Import(C,QVideoCapture_isOpened, "QVideoCapture_isOpened");
 
 procedure QVideoCapture_release(handle:not null  QCVVideoCaptureH);
 pragma Import(C,QVideoCapture_release, "QVideoCapture_release");
 
 function  QVideoCapture_grab(handle:not null  QCVVideoCaptureH) return  boolean;
 pragma Import(C,QVideoCapture_grab, "QVideoCapture_grab");
 
 function  QVideoCapture_retrieve(handle:not null  QCVVideoCaptureH;image: QCVMatH;flag : integer :=  0) return  boolean;
 pragma Import(C,QVideoCapture_retrieve, "QVideoCapture_retrieve");
 
 procedure QVideoCapture_nextFrame(handle:not null  QCVVideoCaptureH;frame: QCVMatH);
 pragma Import(C,QVideoCapture_nextFrame, "QVideoCapture_nextFrame");
 
 procedure QVideoCapture_nextUFrame(handle:not null  QCVVideoCaptureH;frame: QCVUMatH);
 pragma Import(C,QVideoCapture_nextUFrame, "QVideoCapture_nextUFrame");
 
 function  QVideoCapture_read(handle:not null  QCVVideoCaptureH;image: QCVMatH) return  boolean;
 pragma Import(C,QVideoCapture_read, "QVideoCapture_read");
 
 function  QVideoCapture_set(handle:not null  QCVVideoCaptureH;propId:integer;value: qreal) return  boolean;
 pragma Import(C,QVideoCapture_set, "QVideoCapture_set");
 
 function  QVideoCapture_get(handle:not null  QCVVideoCaptureH;propId:integer) return qreal;
 pragma Import(C,QVideoCapture_get, "QVideoCapture_get");
 
 function  QVideoCapture_getBackendName(handle:not null  QCVVideoCaptureH) return  QCVStringH;
 pragma Import(C,QVideoCapture_getBackendName, "QVideoCapture_getBackendName");
 
 function  QVideoCapture_frameWidth(handle:not null QCVVideoCaptureH) return integer;
 pragma Import(C,QVideoCapture_frameWidth, "QVideoCapture_frameWidth");
 
 function  QVideoCapture_frameHeight(handle:not null QCVVideoCaptureH) return integer;
 pragma Import(C,QVideoCapture_frameHeight, "QVideoCapture_frameHeight");
 
 procedure QVideoCapture_setResolution(handle:not null   QCVVideoCaptureH;width,height : qreal);
 pragma Import(C,QVideoCapture_setResolution, "QVideoCapture_setResolution");
 
 function  QVideoCapture_fps(handle:not null  QCVVideoCaptureH)  return qreal;
 pragma Import(C,QVideoCapture_fps, "QVideoCapture_fps");
 
 procedure QVideoCapture_setFps(handle:not null QCVVideoCaptureH;value :qreal);
 pragma Import(C,QVideoCapture_setFps, "QVideoCapture_setFps");
 
 function  QVideoCapture_getIntervalByMaxFps(fps: integer) return integer;
 pragma Import(C,QVideoCapture_getIntervalByMaxFps, "QVideoCapture_getIntervalByMaxFps");
 
 procedure QVideoCapture_close(handle:not null  QCVVideoCaptureH);
 pragma Import(C,QVideoCapture_close, "QVideoCapture_close");
 
 function  QVideoCapture_frames(handle:not null QCVVideoCaptureH) return quint64;
 pragma Import(C,QVideoCapture_frames, "QVideoCapture_frames");

 --------------------------------------------------------------------------------------
 function  QVideoWriter_create return  QCVVideoWriterH;
 pragma Import(C,QVideoWriter_create, "QVideoWriter_create");
 
 function  QVideoWriter_create2(fileName:not null QStringH ;fourcc: integer;fps: qt.double;frameSize:QCVSizeH;isColor : boolean :=  true) return QCVVideoWriterH;
 pragma Import(C,QVideoWriter_create2, "QVideoWriter_create2");
 
 function  QVideoWriter_create3(fileName:not null QStringH ;apiPreference: integer;fourcc: integer;fps: qt.double;frameSize:QCVSizeH;isColor : boolean :=  true) return QCVVideoWriterH;
 pragma Import(C,QVideoWriter_create3, "QVideoWriter_create3");
 
 function  QVideoWriter_create4(fileName:not null QCVStringH ;fourcc: integer;fps: qt.double;frameSize:QCVSizeH;isColor : boolean :=  true) return QCVVideoWriterH;
 pragma Import(C,QVideoWriter_create4, "QVideoWriter_create4");
 
 function  QVideoWriter_create5(fileName:not null QCVStringH ;apiPreference: integer;fourcc: integer;fps: qt.double;frameSize:QCVSizeH;isColor : boolean :=  true) return QCVVideoWriterH;
 pragma Import(C,QVideoWriter_create5, "QVideoWriter_create5");
 

 
 procedure QVideoWriter_destroy(handle:not null  QCVVideoWriterH);
 pragma Import(C,QVideoWriter_destroy, "QVideoWriter_destroy");
 
 function  QVideoWriter_open(handle:not null  QCVVideoWriterH;fileName: QCVStringH;fourcc: integer;fps: qreal;frameSize: QCVSizeH;isColor :  boolean :=  true) return  boolean;
 pragma Import(C,QVideoWriter_open, "QVideoWriter_open");
 
 function  QVideoWriter_open2(handle:not null  QCVVideoWriterH;fileName: QStringH;apiPreference: integer;fourcc: integer;fps: qreal;frameSize: QCVSizeH;isColor :  boolean :=  true) return  boolean;
 pragma Import(C,QVideoWriter_open2, "QVideoWriter_open2");
 
 function  QVideoWriter_isOpened(handle:not null  QCVVideoWriterH) return  boolean;
 pragma Import(C,QVideoWriter_isOpened, "QVideoWriter_isOpened");
 
 procedure QVideoWriter_release(handle:not null  QCVVideoWriterH);
 pragma Import(C,QVideoWriter_release, "QVideoWriter_release");
 
 procedure QVideoWriter_putMat(handle:not null  QCVVideoWriterH;image: QCVMatH);
 pragma Import(C,QVideoWriter_putMat, "QVideoWriter_putMat");
 
 procedure QVideoWriter_putUMat(handle:not null  QCVVideoWriterH;image: QCVUMatH);
 pragma Import(C,QVideoWriter_putUMat, "QVideoWriter_putUMat");
 
 procedure QVideoWriter_write(handle:not null  QCVVideoWriterH;image: QCVMatH);
 pragma Import(C,QVideoWriter_write, "QVideoWriter_write");
 
 function  QVideoWriter_set(handle:not null  QCVVideoWriterH;propId:integer;value: qreal) return  boolean;
 pragma Import(C,QVideoWriter_set, "QVideoWriter_set");
 
 function  QVideoWriter_get(handle:not null  QCVVideoWriterH;propId:integer) return qreal;
 pragma Import(C,QVideoWriter_get, "QVideoWriter_get");
 
 function  QVideoWriter_fourcc(c1: character := 'M';c2: character := 'J';c3: character := 'P';c4: character := 'G') return integer;
 pragma Import(C,QVideoWriter_fourcc, "QVideoWriter_fourcc");
 
 function  QVideoWriter_getBackendName(handle:not null  QCVVideoWriterH) return  QStringH;
 pragma Import(C,QVideoWriter_getBackendName, "QVideoWriter_getBackendName");
 
 function  QVideoWriter_createFromFrame(fileName:QStringH;apiPreference,fourcc: integer;fps: double;frameWidth,frameHeight:integer;isColor : boolean :=  true) return QCVVideoWriterH;
 pragma Import(C,QVideoWriter_createFromFrame, "QVideoWriter_createFromFrame");
 
 function  QVideoWriter_createFromFrame2(fileName:QStringH;fourcc: integer;fps: double;frameWidth,frameHeight:integer;isColor : boolean :=  true) return QCVVideoWriterH;
 pragma Import(C,QVideoWriter_createFromFrame2, "QVideoWriter_createFromFrame2");
 
 function  QVideoWriter_openFromFrame(handle:not null QCVVideoWriterH;fileName:QStringH;fourcc: integer;fps: double;frameWidth,frameHeight:integer;isColor : boolean :=  true) return boolean;
 pragma Import(C,QVideoWriter_openFromFrame, "QVideoWriter_openFromFrame");
 
 function  QVideoWriter_openFromFrame2(handle:not null QCVVideoWriterH;fileName:QStringH;apiPreference: integer;fourcc: integer;fps: double;frameWidth:integer;frameHeight:integer;isColor : boolean :=  true) return boolean;
 pragma Import(C,QVideoWriter_openFromFrame2, "QVideoWriter_openFromFrame2");
 
 function  QVideoCapture_createRtsp(stream:QStringH := s2qs("http://localhost:56000/mjpeg");apiPreference : VideoCaptureAPIs := CAP_FFMPEG) return QCVVideoCaptureH;
 pragma Import(C,QVideoCapture_createRtsp, "QVideoCapture_createRtsp");
 
 type frameCallbackH is access procedure(frame:QCVMatH);
 pragma convention(C,frameCallbackH);

 procedure QVideoCapture_signal_slot_frameChanged(handle:not null QCVVideoCaptureH;hook:frameCallbackH);
 pragma Import(C,QVideoCapture_signal_slot_frameChanged, "QVideoCapture_signal_slot_frameChanged");
 
 ---------------------------------------------------------------------------------------------------
 function  QVideoCapture_create(fileName: QCVStringH;apiPreference : VideoCaptureAPIs := CAP_ANY) return  QCVVideoCaptureH;
 function  QVideoCapture_create(index:integer;apiPreference : VideoCaptureAPIs := CAP_ANY) return  QCVVideoCaptureH;
 function  QVideoCapture_open(handle:not null  QCVVideoCaptureH;index:integer;apiPreference : VideoCaptureAPIs := CAP_ANY) return  boolean;
 
 function  QVideoWriter_create(fileName:not null QStringH ;fourcc: integer;fps: qt.double;frameSize:QCVSizeH;isColor : boolean :=  true) return QCVVideoWriterH;
 function  QVideoWriter_create(fileName:not null QStringH ;apiPreference: integer;fourcc: integer;fps: qt.double;frameSize:QCVSizeH;isColor : boolean :=  true) return QCVVideoWriterH;
 function  QVideoWriter_create(fileName:not null QCVStringH ;fourcc: integer;fps: qt.double;frameSize:QCVSizeH;isColor : boolean :=  true) return QCVVideoWriterH;
 function  QVideoWriter_create(fileName:not null QCVStringH ;apiPreference: integer;fourcc: integer;fps: qt.double;frameSize:QCVSizeH;isColor : boolean :=  true) return QCVVideoWriterH;
 function  QVideoWriter_open(handle:not null  QCVVideoWriterH;fileName: QStringH;apiPreference: integer;fourcc: integer;fps: qreal;frameSize: QCVSizeH;isColor :  boolean :=  true) return  boolean;  

 function  QVideoWriter_createFromFrame(fileName:QStringH;fourcc: integer;fps: double;frameWidth,frameHeight:integer;isColor : boolean :=  true) return QCVVideoWriterH;
 function  QVideoWriter_openFromFrame(handle:not null QCVVideoWriterH;fileName:QStringH;apiPreference: integer;fourcc: integer;fps: double;frameWidth,frameHeight:integer;isColor : boolean :=  true) return boolean;


end;
