--
-- ͻ
--        D E S I G N   E N G I N E E R I N G            DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         QtOpenCVAda - Ada to Qt6 and OpenCV interface              
--                                                                    
--         Author :  Leonid Dulman     2022                           
--  Copyright (C) 2021,2022                                           
-- ͼ
--
--   This library is free software; you can redistribute it and/or modify  *
--   it under the terms of the GNU Library General Public License as       *
--   published by the Free Software Foundation; either version 3 of the    *
--   License, or (at your option) any later version.                       *

with VTK.ImageImport ; use VTK.ImageImport;
with VTK.ImageData   ; use VTK.ImageData;
with Qt.QCVCore      ; use  Qt.QCVCore;
with Qt.QCVMat       ; use  Qt.QCVMat;
with Qt.OpenCV       ; use Qt.OpenCV;
with System.Address_To_Access_Conversions;

package body Qt.QVtkCV is

 type accessByte is access all Unsigned_Char;
 package toByte is new System.Address_To_Access_Conversions(Unsigned_Char); use toByte;

 procedure QVTKCV_Mat2VTK(src: not null QCVMatH;dst : not null vtkImageDataH) is

  importer : vtkImageImportH := vtkImageImport_create;
  size : QCVSizeH;
  begin
  vtkImageImport_SetOutput(importer,vtkDataObjectH(dst));
  vtkImageImport_SetDataSpacing(importer, 1, 1, 1 );
  vtkImageImport_SetDataOrigin(importer, 0, 0, 0 );
  size := QCVMat_size(src);
  vtkImageImport_SetWholeExtent(importer,   0,QCVSize_width(size)-1, 0,QCVSize_height(size)-1, 0, 0 );
  vtkImageImport_SetDataExtentToWholeExtent(importer);
  vtkImageImport_SetDataScalarTypeToUnsignedChar(importer);
  vtkImageImport_SetNumberOfScalarComponents(importer,QCVMat_channels(src));
  vtkImageImport_SetImportVoidPointer(importer,vtkObjectH(QCVMat_data(src)));
  vtkImageImport_Update(importer);

 end;
 
 function  QVtkCV_VTK2Mat(handle: not null vtkImageDataH) return QCVMatH is
 imageWidth,imageHeight,numberOfImageChannels : integer;
 cvType: conversion_type;

 extent: ivector(0 .. 2) := (others => 0); -- Width, Hight, Depth --> Depth is not equal to number of image channels!
 resultingCVMat  : QCVMatH;
 ppixel: void_star;
 pixelp: accessByte;
 begin  
  vtkImageData_GetDimensions(handle,extent);
  imageWidth            := extent(0);
  imageHeight           := extent(1);
  numberOfImageChannels := vtkImageData_GetNumberOfScalarComponents(handle);
  cvType := 0;
  case(numberOfImageChannels) is
    when 1 => cvType := CV_8UC1;
    when 3 => cvType := CV_8UC3;
    when 4 => cvType := CV_8UC4;
    when others =>  return QCVMat_create;
  end case;
  resultingCVMat := QCVMat_create(imageHeight, imageWidth, integer(cvType));
  --   Loop over the vtkImageData contents.
  for heightPos in 0 .. imageHeight - 1 loop
    for  widthPos in 0 .. imageWidth - 1 loop
      if vtkImageData_GetScalarPointerForExtent(handle,extent) /= null then null; end if;
	  
      ppixel := vtkImageData_GetScalarPointer(handle,widthPos, heightPos, 0);
	  pixelp := accessByte(To_Pointer(ppixel));
      QCVMat_changePixelAt(resultingCVMat,heightPos, widthPos,pixelp.all);
    end loop;
  end loop;
  return resultingCVMat;
 end;

 
end;
